#include <WiFi.h>
#include <PubSubClient.h>

WiFiClient client;
PubSubClient mqtt_client(client);

//請修改以下參數
//-------------------------------------------------------------------------------------------
char  szSSID[] = "Your_WiFi_AP_SSID";         //Your_SSID
char  szPassword[] = "Your_WiFi_AP_Password"; //Your_Password
char  szMQTTBroker[] = "Your.MQTT.Broker";    //Your_MQTT_Broker
int   nMQTTBrokerPort = 1883;
char  szMQTTClientID[] = "Your_MQTT_Client_ID";         //Unique_MQTT_Client_ID
char  szMQTTSubscribeTopic[] = "Unique_BlueLED_Topic";  //Your_BlueLED_Topic
//-------------------------------------------------------------------------------------------

int   nPublishCnt=1;

float fnUltraSonicDistance(int nTrigPin, int nEchoPin) 
{
    digitalWrite(nTrigPin, LOW);
    digitalWrite(nEchoPin, LOW);
    delayMicroseconds(5);
    digitalWrite(nTrigPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(nTrigPin, LOW);
    unsigned long sonic_duration = pulseIn(nEchoPin, HIGH);
    float fDistanceCM = (sonic_duration / 2.0) / 29.1;
    return fDistanceCM;
}

void fnWiFiConnect()
{
    //連線到指定的WiFi szSSID
    Serial.print("Connecting Wifi: ");
    Serial.println(szSSID);
    WiFi.begin(szSSID, szPassword);
    while (WiFi.status() != WL_CONNECTED) { 
        Serial.print("."); 
        delay(300);
    };
    //連線成功，顯示取得的IP
    digitalWrite(3, LOW); 
    Serial.println("\nWiFi Connected!");
    Serial.print("IP address: ");
    IPAddress ip = WiFi.localIP();
    Serial.println(ip);
}

void fnMQTTConnectCheck() 
{
    while (!mqtt_client.connected()) 
    {
        bool bConnected=false;

        Serial.print(".");
        bConnected = mqtt_client.connect(szMQTTClientID);
        if (bConnected) {   
            mqtt_client.subscribe(szMQTTSubscribeTopic);
            Serial.println("\nMQTT Broker Connected!");
        }
    }
}

void setup() 
{
    Serial.begin(115200);
    
    pinMode(3, OUTPUT);
    pinMode(9, OUTPUT);
    pinMode(11,OUTPUT);
    pinMode(12,INPUT);
    digitalWrite(3, HIGH); 
    digitalWrite(9, HIGH); 
    
    fnWiFiConnect();
    mqtt_client.setCallback(callback);
    
    fnMQTTConnectCheck();
}

void loop() 
{
    fnMQTTConnectCheck();
    mqtt_client.loop();

    float  fDistance=fnUltraSonicDistance(11, 12);
    //Serial.println(fDistance);
    if(fDistance > 0 && fDistance < 20)
    {
        digitalWrite(3, HIGH);
        
        delay(3000);
        digitalWrite(3, LOW);
    }
}

String fnGetMQTTData(String szSubscribeTopic, String szGetTopic, byte* pGetPayload, unsigned int nGetLens)
{
    String szMQTTPayload="";

    if(szGetTopic == szSubscribeTopic){
        for (int i = 0; i < nGetLens; i++) {
            szMQTTPayload += (char)pGetPayload[i];
        }
        return szMQTTPayload;
    }else{
        return "";
    }
}

void callback(char* szGetTopic, byte* pGetPayload, unsigned int nGetLens)
{
    Serial.println((fnGetMQTTData(szMQTTSubscribeTopic, szGetTopic, pGetPayload, nGetLens)));
    if (fnGetMQTTData(szMQTTSubscribeTopic, szGetTopic, pGetPayload, nGetLens) == "ON") {
        digitalWrite(9, LOW);
    } 
    else if (fnGetMQTTData(szMQTTSubscribeTopic, szGetTopic, pGetPayload, nGetLens) == "OFF") {
        digitalWrite(9, HIGH);
    }
}
