#include <WiFi.h>
#include <WiFiUdp.h>
#include <NTPClient.h>

//請修改以下參數
//-------------------------------------------------------------------------------------------
char    szSSID[] = "Your_WiFi_AP_SSID";
char    szPassword[] = "Your_WiFi_AP_Password";
String  szLINEToken = "Your_LINE_Notify_Token";
String  szAPIKey = "Your_LimeWire_API_Key";
String  szGeminiAPIKey = "Your_Gemini_API_Key";
//水中開出很多蓮花，翠綠的蓮葉與粉紅的蓮花上還帶有清晨的露珠。
String  szImagePrompt="There are many lotus flowers blooming in the water, and the green lotus leaves and pink lotus flowers are still covered with morning dew.";
String  szGeminiPrompt="請用繁體中文寫出一句祝福的吉祥話";
String  szBookHour="11", szBookMin="24";
//-------------------------------------------------------------------------------------------

String  szLimeWireReply="";
String  szGeminiProMsg="";

WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP);

float fnUltraSonicDistance(int nTrigPin, int nEchoPin) 
{
    digitalWrite(nTrigPin, LOW);
    digitalWrite(nEchoPin, LOW);
    delayMicroseconds(5);
    digitalWrite(nTrigPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(nTrigPin, LOW);
    unsigned long sonic_duration = pulseIn(nEchoPin, HIGH);
    float fDistanceCM = (sonic_duration / 2.0) / 29.1;
    return fDistanceCM;
}

void fnWiFiConnect()
{
    //連線到指定的WiFi szSSID
    Serial.print("Connecting Wifi: ");
    Serial.println(szSSID);
    WiFi.begin(szSSID, szPassword);
    while (WiFi.status() != WL_CONNECTED) { 
        Serial.print("."); 
        delay(300);
    };
    //連線成功，顯示取得的IP
    digitalWrite(3, LOW); 
    Serial.println("\nWiFi Connected!");
}

bool fnLimeWireT2P(String szRequest, String szRatio, String szLimeWireKey) 
{
    WiFiSSLClient client_tcp;

    szLimeWireReply="";
    szRequest = "{\"prompt\":\"" + szRequest + "\",\"aspect_ratio\":\"" + szRatio + "\"}";
    if (client_tcp.connect("api.limewire.com", 443)) {
        client_tcp.println("POST /api/image/generation HTTP/1.1");
        client_tcp.println("Host: api.limewire.com");
        client_tcp.println("Authorization: Bearer " + szLimeWireKey);
        client_tcp.println("Content-Type: application/json; charset=utf-8");
        client_tcp.println("Content-Length: " + String(szRequest.length()));
        client_tcp.println();
        client_tcp.println(szRequest);
        
        int nTimeout = 30000;
        long lCurTime = millis(), j=0;
        String szResponse="", szFeedback="";
        while ((lCurTime + nTimeout) > millis()) {
            Serial.print(".");
            delay(500);
            while (client_tcp.available()) {
                szResponse=client_tcp.readStringUntil('\n');
                //Serial.println(szResponse);
                if(szResponse.indexOf("asset_url")>=0){
                    Serial.println();
                    szFeedback=szResponse.substring(szResponse.indexOf("\"asset_url\":\"")+13, szResponse.lastIndexOf("\",\"type\""));
                    szLimeWireReply=szFeedback;
                    break;
                }
                else if(szResponse.indexOf("detail")>=0){
                    Serial.println();
                    szFeedback=szResponse.substring(szResponse.indexOf("\"detail\":\"")+10, szResponse.lastIndexOf("\",\"type\""));
                    szLimeWireReply=szFeedback;
                    break;
                }
                lCurTime = millis();
            }
            if(szLimeWireReply!="") {
                Serial.println(szLimeWireReply);
                break;
            }
        }
        client_tcp.flush();
        client_tcp.stop();
        if (szLimeWireReply.indexOf("https")>=0)
            return true;
        else if (szLimeWireReply.length()>0) {
            szLimeWireReply="";
            return false;
        }
        else {
            Serial.println("Timeout Error!");
            return false;
        }
    }
    else{
        Serial.println("Connect to LimeWire Fail!");
        return false;
    }
}

String fnGeminiProT2T(String szRequest, int nMaxTokens, String szAPIKey) 
{ 
    WiFiSSLClient clientSecure;

    szRequest = "{\"contents\": [{\"parts\":[{\"text\":\"" + szRequest + "\"}]}]}";
    if (clientSecure.connect("generativelanguage.googleapis.com", 443)) {
        clientSecure.println("POST /v1beta/models/gemini-pro:generateContent?key=AIzaSyCY7VtYJ8eVrbBVdvl9GPLdURy5VwABJmA HTTP/1.1");
        clientSecure.println("Host: generativelanguage.googleapis.com");
        clientSecure.println("Content-Type: application/json; charset=utf-8");
        clientSecure.println("Content-Length: " + String(szRequest.length()));
        clientSecure.println();
        clientSecure.println(szRequest);

        int nTimeout = 30000;
        long lCurTime = millis();
        String szResponse="",szFeedback="";
        while ((lCurTime + nTimeout) > millis()) {
            Serial.print(".");
            delay(500);
            while (clientSecure.available()) {
                szResponse=clientSecure.readStringUntil('\n');
                if(szResponse.indexOf("\"text\": \"")>=0){
                    Serial.println();
                    szResponse=szResponse.substring(szResponse.indexOf("\"text\": \"")+9, szResponse.lastIndexOf("\""));
                    szFeedback=szResponse;
                    break;
                }else if(szResponse.indexOf("API key not valid")>=0){
                    Serial.println();
                    szFeedback="Incorrect API key.";
                    break;
                }
                lCurTime = millis();
            }
            if (szFeedback.length()>0)
                break;
        }
        clientSecure.flush();
        clientSecure.stop();
        if (szFeedback.length()>0)
            return szFeedback;
        else
            return "Request timeout!";
    }
    else
    {
        clientSecure.flush();
        clientSecure.stop();
        return "GeminiPro connection failed!";
    }
}

void setup() 
{
    Serial.begin(115200);
    
    pinMode(3, OUTPUT);
    pinMode(9, OUTPUT);
    pinMode(11,OUTPUT);
    pinMode(12,INPUT);
    digitalWrite(3, HIGH); 
    digitalWrite(9, HIGH); 
    
    fnWiFiConnect();

    timeClient.begin();                // NTP  連接 
    timeClient.setTimeOffset(28800);   // GMT +8 = 28800   台灣時區 
    szGeminiProMsg = fnGeminiProT2T(szGeminiPrompt, 128, szGeminiAPIKey);
    Serial.println(szGeminiProMsg);
}

void loop() 
{
    if(WiFi.status() != WL_CONNECTED)
        fnWiFiConnect();

    timeClient.update();
        
    float   fDistance=fnUltraSonicDistance(11, 12);
    int     nCurHour=timeClient.getHours(), nCurMin=timeClient.getMinutes();

    if((fDistance > 0 && fDistance < 20) || (szBookHour==String(nCurHour) && szBookMin==String(nCurMin)))
    {
        digitalWrite(3, HIGH);
        fnLimeWireT2P(szImagePrompt, "1:1", szAPIKey);
        szGeminiProMsg="早安，祝您：" + szGeminiProMsg + "！";
        fnLINENotifyImage(szGeminiProMsg, szLINEToken, szLimeWireReply, szLimeWireReply);
        szGeminiProMsg = fnGeminiProT2T(szGeminiPrompt, 128, szGeminiAPIKey);
        Serial.println(szGeminiProMsg);
        if(szBookHour==String(nCurHour) && szBookMin==String(nCurMin))
            delay(55000);
        digitalWrite(3, LOW);
    }
}

String URLEncode(const char* msg)
{
    const char *hex = "0123456789abcdef";
    String encodedMsg = "";
    while (*msg!='\0'){
        if(('a' <= *msg && *msg <= 'z')||('A' <= *msg && *msg <= 'Z')||('0' <= *msg && *msg <= '9')){
            encodedMsg += *msg;
        }else{
            encodedMsg += '%';
            encodedMsg += hex[*msg >> 4];
            encodedMsg += hex[*msg & 15];
        }
        msg++;
    }
    return encodedMsg;
}

void fnLINENotifyImage(String szLINEMsg, String szToken, String szThumbnailURL, String szFullSizeURL) 
{
    WiFiSSLClient clientSecure;

    if (clientSecure.connect("notify-api.line.me", 443)) {
        String data = "message="+URLEncode(szLINEMsg.c_str())+"&imageThumbnail="+URLEncode(szThumbnailURL.c_str())+"&imageFullsize="+URLEncode(szFullSizeURL.c_str());
        
        clientSecure.println("POST /api/notify HTTP/1.1");
        clientSecure.println("Host: notify-api.line.me");
        clientSecure.println("Authorization: Bearer "+ szToken);
        clientSecure.println("Content-Type: application/x-www-form-urlencoded");
        clientSecure.print("Content-Length: ");
        clientSecure.println(data.length());
        clientSecure.println();
        clientSecure.println(data);
        clientSecure.stop();
    }
}